@@ -85,43 +85,43 @@ class BaseEnvironnement(BaseEstimator):
     Notes
     -----
     Les attributs constituant BaseEnvironnement :
-        _is_fitted : bool
+        _is_fitted: bool
             détermine si le modèle est entraîné
 
     Les méthodes constituant BaseEnvironnement :
-        number_of_parameters : 
+        number_of_parameters:
             nombre de paramètre (entraînable ou non) constiuant la 
             modélisation
-        save_model : 
+        save_model:
             sauvegarde du modèle via Pickle
-        load_weights : 
+        load_weights:
             chargement des poids via Pickle. Si modèle non buildé, build le 
             modèle
-        load_model :
+        load_model:
             chargement d'un modèle à partir du Pickle au format save_weights
-        fit :
+        fit:
             fonction d'apprentissage avec prototypage compatible avec 
             Scikit-Learn
 
     Les méthodes qui doivent être implémentées dans la classe du réseau :
-        __init__ : 
+        __init__:
             avec seulement les hyper-paramètres du réseau, afin de pouvoir 
             modifier les hyper-paramètres avec la méthode set_params de 
             BaseEstimator.
-        build : 
+        build:
             fonction d'implémentation des éléments du réseau PyTorch ainsi 
             que de l'attribut optimizer contenant l'optimizer du réseau
-        forward :
+        forward:
             fonction PyTorch d'éxécution du réseau
-        predict :
+        predict:
             fonction de prédiction, au prototypage Scikit-Learn 
             predict(X, **kargs) :
                 - classifier : doit sortir une classe
                 - regresseur : doit sortir une régression
-        predict_proba : (optionnel)
+        predict_proba: (optionnel)
             pour les classifier uniquement. Permet de sortir la probabilité 
             d'une observation dans chaqu'une des classes
-        score : (optionnel) 
+        score: (optionnel)
             au format de prototype Scikit-Learn. Si cette méthode n'est pas 
             implémentée par l'utilisateur, ce sont alors les méthodes par 
             défaut dans Scikit-Learn qui seront utilisées, à savoir dans le 
@@ -174,10 +174,10 @@ class BaseEnvironnement(BaseEstimator):
 
     def save_model(
         self,
-        path: Optional[str]=None,
+        path: Optional[str] = None,
         *,
-        save_optimizer_state: bool=False,
-        attr_list: Optional[Union[List[str], Tuple[str]]]=None,
+        save_optimizer_state: bool = False,
+        attr_list: Optional[Union[List[str], Tuple[str]]] = None,
         **kargs_dump
     ):
         r"""
@@ -186,14 +186,14 @@ class BaseEnvironnement(BaseEstimator):
 
         Parameters
         ----------
-        path : Optional[str]
+        path: Optional[str]
             path directory du modèle, si None retourne les données binaires de 
             Pickle.dumps()
-        save_state_optimizer : bool
+        save_state_optimizer: bool
             booléen de sauvegarde de l'état de l'optimizer (par défaut False).
-        attr_list : Optional[Union[List[str], Tuple(str)]]
+        attr_list: Optional[Union[List[str], Tuple(str)]]
             liste d'attributs à sauvegarder (par défaut None).
-        **kargs_dump : 
+        **kargs_dump:
             paramètres attachés à Pickle.dump(obj, file, **kargs_dump) si 
             path est un chemin ou a Pickles.dump(obj, **kargs_dump) si None.
         """
@@ -254,9 +254,9 @@ class BaseEnvironnement(BaseEstimator):
 
         Parameters
         ----------
-        path_or_bytes : Union[str, ByteString]
+        path_or_bytes: Union[str, ByteString]
             path directory du modèle ou bytes représentant le modèles
-        **kargs_load :
+        **kargs_load:
             paramètres attachés à Pickle.load(file, **kargs_load) si path ou 
             Pickle.loads(data, **kargs_load) si bytes.
         """
@@ -302,15 +302,15 @@ class BaseEnvironnement(BaseEstimator):
 
         Parameters
         ----------
-        path_or_bytes : Union[str, ByteString]
+        path_or_bytes: Union[str, ByteString]
             path directory du modèle ou bytes représentant le modèles
-        **kargs_load : 
+        **kargs_load:
             paramètres attachés à Pickle.load(file, **kargs_load) si path ou 
             Pickle.loads(data, **kargs_load) si bytes.
 
         Returns
         -------
-        model : BaseEnvironnement
+        model: BaseEnvironnement
             modèle chargé
         """
         if isinstance(path_or_bytes, str):
@@ -351,7 +351,7 @@ class BaseEnvironnement(BaseEstimator):
 
         Parameters
         ----------
-        model :
+        model:
             un modèle
         """
         return OrderedDict(
@@ -392,26 +392,26 @@ class BaseEnvironnement(BaseEstimator):
 
         Parameters
         ----------
-        data : List[Dict[str, Any]]
+        data: List[Dict[str, Any]]
             Liste de dictionnaires sur les données d'un mini-batch.
-        loss_fn : Callable
+        loss_fn: Callable
             Fonction coût PyTorch.
-        loss_kargs : Dict[str, Any]
+        loss_kargs: Dict[str, Any]
             Dictionnaire d'arguments à utiliser dans la fonction de coût 
             loss_fun.
-        state : FitState
+        state: FitState
             Etat de l'apprentissage (instancié dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        control : FitControl
+        control: FitControl
             Contrôle de l'apprentissage (instancié dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        list_callbacks : Optional[List[CallbackInterface]]
+        list_callbacks: Optional[List[CallbackInterface]]
             Liste des callback permettant de contôler l'apprentissage durant 
             la phase d'évaluation.
 
         Returns
         -------
-        loss : Valeur de la fonction coût au travers du mini-batch.
+        loss: Valeur de la fonction coût au travers du mini-batch.
         """
         if list_callbacks:
             list_callbacks.begin_step(self, data, state, control, None)
@@ -461,20 +461,20 @@ class BaseEnvironnement(BaseEstimator):
 
         Parameters
         ----------
-        data : List[Dict[str, Any]]
+        data: List[Dict[str, Any]]
             Liste de dictionnaires sur les données d'un mini-batch.
-        loss_fn : Callable
+        loss_fn: Callable
             Fonction coût PyTorch.
-        loss_kargs : Dict[str, Any]
+        loss_kargs: Dict[str, Any]
             Dictionnaire d'arguments à utiliser dans la fonction de coût 
             loss_fun.
-        control : FitControl
+        control: FitControl
             Contrôle de l'apprentissage (instancé dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
 
         Returns
         -------
-        loss : Valeur de la fonction coût au travers du mini-batch.
+        loss: Valeur de la fonction coût au travers du mini-batch.
         """
         if control.device.type != "cpu":
             for key, value in data.items():
@@ -511,26 +511,26 @@ class BaseEnvironnement(BaseEstimator):
 
         Parameters
         ----------
-        eval_dataset : DataLoader
+        eval_dataset: DataLoader
             Dataset d'évaluation au format DataLoeader généré dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`.
-        loss_fn : Callable
+        loss_fn: Callable
             Fonction coût à minimiser (utiliser les fonctions présentes dans 
             torch.nn.functional). Si la fonction est une fonction custom, elle 
             doit avoir le prototypage suivant :
                 - loss_fn(y_estim, y_true, reduction='sum', **kargs)
             avec une implémentation de réduction 'sum' pour les phases 
             d'évaluation et 'mean' pour les phases d'apprentissage.
-        loss_kargs : Optional[Dict[str, Any]]
+        loss_kargs: Optional[Dict[str, Any]]
             Dictionnaire d'arguments à utiliser dans la fonction de coût 
             loss_fun.
-        state : FitState
+        state: FitState
             Etat de l'apprentissage (instancié dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        control : FitControl
+        control: FitControl
             Contrôle de l'apprentissage (instancé dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        list_callbacks : Optional[List[CallbackInterface]]
+        list_callbacks: Optional[List[CallbackInterface]]
             Liste des callback permettant de contôler l'apprentissage durant 
             la phase d'évaluation.
         """
@@ -574,64 +574,66 @@ class BaseEnvironnement(BaseEstimator):
     def fit(
         self,
         X: Union[List[Dict[str, Any]], Dataset, DataLoader],
-        y: Optional[List[Union[int, float]]]=None,
+        y: Optional[List[Union[int, float]]] = None,
         *,
-        eval_dataset: Optional[DataLoader]=None,
-        target_field: str='target',
-        target_checktensor: bool=True,
+        eval_dataset: Optional[DataLoader] = None,
+        target_field: str = 'target',
+        target_checktensor: bool = True,
         loss_fn: _Loss,
-        loss_kargs: Dict[str, Any]=dict(),
-        optimizer: Optional[Optimizer]=None,
-        optimizer_kargs: Dict[str, any]=dict(),
+        loss_kargs: Dict[str, Any] = dict(),
+        optimizer: Optional[Optimizer] = None,
+        optimizer_kargs: Dict[str, any] = dict(),
         nb_epoch: int,
-        iter_eval: Optional[int]=None,
-        callbacks: Optional[List[CallbackInterface]]=None,
-        device: str='cuda:0',
-        mixed_type_gpu: bool=False,
-        expected_key_to_device: List[str]=list(),
-        gradient_accumulation: Optional[int]=None,
-        max_gradient_norm: Optional[float]=None,
-        dataloader_kargs: Dict[str, Any]=dict(),
-        n_jobs: Optional[int]=None,
-        state: Optional[FitState]=None,
-        control: Optional[FitControl]=None,
-        build: bool=True
+        iter_eval: Optional[int] = None,
+        callbacks: Optional[List[CallbackInterface]] = None,
+        device: str = 'cuda:0',
+        mixed_type_gpu: bool = False,
+        expected_key_to_device: List[str] = list(),
+        gradient_accumulation: Optional[int] = None,
+        max_gradient_norm: Optional[float] = None,
+        dataloader_kargs: Dict[str, Any] = dict(),
+        begin_apply_fn: Callable = lambda self: None,
+        end_apply_fn: Callable = lambda self: None,
+        n_jobs: Optional[int] = None,
+        state: Optional[FitState] = None,
+        control: Optional[FitControl] = None,
+        build: bool = True
     ):
         r"""
         Méthode de fitting généralisé du module PyTorch.
-        
+
         Parameters
         ----------
-        X : Union[List[Dict[str, Any]], Dataset, DataLoader]
+        X: Union[List[Dict[str, Any]], Dataset, DataLoader]
             Dataset PyTorch contenant dans un dictionnaire les infos 
             nécessaires pour l'apprentissage du modèle.
-        y : Optional[List[Union[int, float]]] 
+        y: Optional[List[Union[int, float]]] 
             Pour être ISO avec Scikit-learn. L'ensemble de l'apprentissage
             passera via X et le système DataSet et DataLoader de PyTorch.
-        eval_dataset : Optional[DataLoader]
+        eval_dataset: Optional[DataLoader]
             DataLoader PyTorch contenant les observations et les targets 
             d'évaluation. Si eval_dataset est None alors pas de early stopping 
             et évaluation des performances sur les données d'entrainement 
             (déconseillé car très fort risque d'overfitting).
-        target_field : str (='target')
+        target_field: str (='target')
             Nom du champ contenu dans X de la target. Par défaut "target". 
             Attention l'implémentation de la foction collate_fn, peut faire 
             changer le nom de la target.
         target_checktensor: bool (=True)
             Permet de checker si le champs target est un tensor, sinon le
             converti. Si False ne checke pas si le champs est un tensor.
-        loss_fn : _Loss
+        loss_fn: _Loss
             Fonction coût à minimiser (utiliser les fonctions présentes dans 
             torch.nn.functional). Si la fonction est une fonction custom, elle 
             doit avoir le prototypage suivant :
                 - loss_fn(y_estim, y_true, reduction='sum', **kargs)
             avec une implémentation de réduction 'sum' pour les phases 
             d'évaluation et 'mean' pour les phases d'apprentissage.
-        loss_kargs : Dict[str, Any]
+        loss_kargs: Dict[str, Any]
             Dictionnaire d'arguments à utiliser dans la fonction decoût 
             loss_fun. Si il n'y a pas d'arguments, mettre un dictionnaire 
             vide.
-        optimizer : Optional[Optimizer]
+        optimizer: Optional[Optimizer]
             Objet optimizer de PyTorch. Si non renseigné doit être implémenté 
             dans le modèle et dans la partie build. Utile si un paramètre de 
             l'optimizer doit faire partie des hyperparamètres. Sinon à 
@@ -643,39 +645,49 @@ class BaseEnvironnement(BaseEstimator):
             Paramètres pour l'optimizer, si optimizer inclue dans la 
             modélisation et l'argument optimizer est à None cet argument ne 
             sera pas pris en compte.
-        nb_epoch : int (=5)
+        nb_epoch: int (=5)
             Nombre d'époques max de l'apprentissage.
-        iter_eval : Optional[int]
+        iter_eval: Optional[int]
             Nombre d'itération entre deux évaluations. Actif seulement si 
             eval_set est non None. Si iter_eval est à None (définit par 
             défaut) les évaluations du modèle seront à la fin des époques. 
             Si un entier positif est renseigné les évaluations auront lieu 
             tous les iter_eval itérations d'apprentissage.
-        callbacks : Optional[List[CallbackInterface]]
+        callbacks: Optional[List[CallbackInterface]]
             Liste de callback intervenant aux différentes étapes de 
             l'entraînement et de l'évaluation. Pour avoir la liste des 
             différents callbacks implémentés par défaut voir 
             :func:`~dstk.pytorch._callback`.
-        device : str (='cuda:0')
+        device: str (='cuda:0')
             Indique le device sur le quel l'apprentissage s'exécute.
-        mixed_type_gpu : bool (=False)
+        mixed_type_gpu: bool (=False)
             Utilisation de nVidia AMP (Automatic Mixed Precision) pour GPU 
             permettant le typage mixte FP16 et FP32.
-        expected_key_to_device : List[str] (=list())
+        expected_key_to_device: List[str] (=list())
             Liste des variables à exclure vers la charge sur GPU.
-        gradient_accumulation : Optional[int]
+        gradient_accumulation: Optional[int]
             Nombre d'itération d'entrainement avec back propagation du 
             gradient avant d'appliquer la descente de gradient.
-        max_gradient_norm : Optional[float]
+        max_gradient_norm: Optional[float]
             Valeur maximum du gradient, sinon clipping de ce dernier par la 
             valeur max_gradient_norm. Si non renseigné alors pas de gradient 
             clipping. Le clipping du gradient peut être utilie pour l'approche 
             par accumulation des gradient.
-        dataloader_kargs : Dict[str, Any] (=dict())
+        dataloader_kargs: Dict[str, Any] (=dict())
             Si X n'est pas un objet Dataloader permet d'entrer les arguments 
             du Dataloader (voir la `documention PyTorch_ `pour connaitre les 
             arguments du Dataloader).
-        n_jobs : Optional[int]
+        begin_apply_fn: Callable (=lambda: None)
+            Fonction appelé avant l'entrainement. Peut faire référence à
+            self à l'intérieur et permettre de changer un comportement du
+            modèle au moment de l'appel de la méthode fit par un autre module
+            (RandomizedSearchCV de Scikit-Learn par exemple).
+        end_apply_fn: Callable (=lambda: None)
+            Fonction appelé à la fin de l'entrainement. Peut faire référence à
+            self à l'intérieur et permettre de changer un comportement du
+            modèle au moment de l'appel de la méthode fit par un autre module
+            (RandomizedSearchCV de Scikit-Learn par exemple).
+        n_jobs: Optional[int]
             Nombre de process des étapes d'inférence et de backpropagation des 
             gradients :
                 - Si y est not None :
@@ -685,13 +697,13 @@ class BaseEnvironnement(BaseEstimator):
                     (et pas le nombre de threads) ;
                 - Si un entier :
                     utilise le nombre de processus indiqué.
-        state : Optional[FitState]
+        state: Optional[FitState]
             Etat de l'apprentissage (instancié dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        control : Optional[FitControl]
+        control: Optional[FitControl]
             Contrôle de l'apprentissage (instancé dans la méthode 
             :func:`~dstk.pytorch._base.BaseEnvironnement.fit`).
-        build : bool (=True)
+        build: bool (=True)
             Permet de reconstruire le réseau si, entre la définition de la 
             classe et le fit, il y a changement d'hyper-paramètres (exemple : 
             RandomizedSearchCV ou toute autre fonction Scikit-learn pouvant 
@@ -831,7 +843,7 @@ class BaseEnvironnement(BaseEstimator):
 
         # Si y est non None c'est que le processes passe par un search de
         # Scikit-Learn.
-        if y is not None:
+        if y is not None and n_jobs is None:
             torch.set_num_threads(1)
             dataset.num_workers = 0
         elif n_jobs is not None:
@@ -907,6 +919,7 @@ class BaseEnvironnement(BaseEstimator):
         elif len(state_in_loss_kargs) == 1:
             loss_kargs['state'] = state
 
+        begin_apply_fn(self)
         list_callbacks.begin_train(self, dataset, state, control, None)
         try:
             for n_epoch in range(nb_epoch):
@@ -992,4 +1005,5 @@ class BaseEnvironnement(BaseEstimator):
         list_callbacks.end_train(self, dataset, state, control, None)
         if control.device.type != "cpu": self.to(torch.device('cpu:0'))
         self._is_fitted = True
+        end_apply_fn(self)
         return self
