@@ -7,12 +7,15 @@ Created on Thu Oct 31 09:42:49 2019
 
 @author: Cyrile Delestre
 """
+
+import random
 from typing import Union, Optional, Tuple, List, Type, Dict, Any
 
 import numpy as np
 from pandas.core.frame import DataFrame
 
 import torch
+from torch.utils.data import get_worker_info
 import torch.distributed as dist
 
 from dstk.utils.deprecated import deprecated
@@ -20,8 +23,9 @@ from dstk.utils.errors import CheckError
 
 
 def check_tensor(
-        array: Union[torch.Tensor, np.ndarray, DataFrame, List, Tuple, int, float],
-        dtype: Optional[Type]=None
+        array: Union[torch.Tensor, np.ndarray, DataFrame,
+                     List, Tuple, int, float],
+        dtype: Optional[Type] = None
     ):
     r"""
     Fonction vérifiant si le type de l'entrée est bien transformable en Tensor 
@@ -221,9 +225,9 @@ def detach(array):
 def collate_zeros_padding(
     sample: List[Dict[str, Union[torch.Tensor, np.ndarray]]],
     fields_series: List[Union[List, Tuple]],
-    padding_value: float=0,
-    torch_type: bool=True,
-    batch_first: bool=True
+    padding_value: float = 0,
+    torch_type: bool = True,
+    batch_first: bool = True
 ):
     r"""
     Fonction de padding générique pour des séquences de différente longueur.
@@ -238,17 +242,17 @@ def collate_zeros_padding(
         de  tous les arrays présente dans toutes les listes. Convertie ensuite 
         tous les champs en tensor. Exemple de sample :
             [{'a': np.array, 'b': np.array, 'siz_a': int, ...}, ...]
-    fields_series : List[Union[List, Tuple]]
+    fields_series: List[Union[List, Tuple]]
         liste de liste contenant le nom du champ de la série et 
         optionnellement le champ de la taille qui lui est associé. S'il n'y a 
         pas de champs associé à taille alors prend le maximum de la taille. 
         Par exemple avec l'exemple précédent :
             [['a', 'siz_a'], ['b']] ou  encore [['a', siz_a], ['b', None]]
-    padding_value : float
+    padding_value: float
         valeur des valeurs ajoutées. Par défaut 0.
-    torch_type : bool
+    torch_type: bool
         retourne les vecteurs au format PyTorch (True) ou Numpy (False).
-    batch_first : bool
+    batch_first: bool
         défini sur la première dimension est celle du batch (la seconde est 
         alors la série) ou l'inverse. Attention dans PyTorch par défaut dans 
         les RNN c'est l'inverse.
@@ -261,7 +265,7 @@ def collate_zeros_padding(
     
     Returns
     -------
-    batch : Dict[str, Union[Tensor, np.ndarray]]
+    batch: Dict[str, Union[Tensor, np.ndarray]]
         dict au format :
             - {'a': tensor, 'b': tensor, 'siz_a': tensor, ...}
     """
@@ -387,7 +391,7 @@ def make_mask(
         n_channel: int,
         shift: int,
         kernel_size: int,
-        repeat: Optional[Union[List, Tuple]]=None
+        repeat: Optional[Union[List, Tuple]] = None
 ) -> torch.BoolTensor:
     r"""
     Fonction de création de masque permettant de personaliser la topologie 
@@ -395,23 +399,23 @@ def make_mask(
     
     Parameters
     ----------
-    in_features : int
+    in_features: int
         taille en entré du réseau
-    out_features : int
+    out_features: int
         taille en sortie du réseau
-    n_channel : int
+    n_channel: int
         nombre de neuronnes voyant les mêmes entrées
-    shift : int
+    shift: int
         décalage incrémentale sur chaques neuronnes
-    kernel_size : int
+    kernel_size: int
         entrée contigue en entré du neuronne
-    repeat : Optional[Union[List, Tuple]] (=None)
+    repeat: Optional[Union[List, Tuple]] (=None)
         tuple ou liste permettant de constituer les connexions (utile si 
         entrés multivariées).
     
     Returns
     -------
-    mask : BoolTensor
+    mask: BoolTensor
         masque de dimension (out_feature x repeat[0]), 
         (in_features x repeat[1])
     
@@ -445,6 +449,22 @@ def make_mask(
     return mask
 
 
+def worker_init_4_dataloader(worker_id: int):
+    r"""
+    Initialisation de workers du DataLoader permettant de garentir que
+    dans chaque worker les générateurs aléatoires Python et Numpy seront
+    différent et initialisé par la seed PyTorch du worker.
+
+    Parameters
+    ----------
+    worker_id: int
+        id du worker -> worker_id == get_worker_info().id
+    """
+    seed = get_worker_info().seed + worker_id
+    np.random.seed(seed % 2**32)
+    random.seed(seed)
+
+
 def is_dist_avail_and_initialized():
     r"""
     Détermine si on est dans la cas d'un process distribué ou non.
