@@ -11,6 +11,7 @@ from ._transpose import (list_of_dict_2_dict_of_list,
                          dict_of_list_2_list_of_dict)
 from ._rolling import RollingWindow
 from ._trie import Trie
+from ._seed import set_seed
 
 __all__ = (
     'Chunker',
@@ -26,5 +27,6 @@ __all__ = (
     'list_of_dict_2_dict_of_list',
     'dict_of_list_2_list_of_dict',
     'RollingWindow',
-    'Trie'
+    'Trie',
+    'set_seed',
 )
