@@ -11,6 +11,7 @@ Created on Sat Mar 27 10:17:24 2021
 
 from typing import List, Union, Any
 from dataclasses import dataclass
+from copy import deepcopy
 
 import numpy as np
 import pandas as pd
@@ -24,7 +25,7 @@ class RollingWindow:
 
     Parameters
     ----------
-    data : Union[List[Any], np.ndarray, pd.DataFrame]
+    data : Union[List[Any], str, np.ndarray, pd.DataFrame]
         Itérable où sera appliquer le rolling.
     window : int
         Taille de la fenêtre du rolling. Elle doit être au minimum de taille 
@@ -62,12 +63,12 @@ class RollingWindow:
     >>> roll[4]
     IndexError: Index out of band, l'index doit être compris entre 0 et 3.
     """
-    data: Union[List[Any], np.ndarray, pd.DataFrame]
+    data: Union[List[Any], str, np.ndarray, pd.DataFrame]
     window: int
     position: str = 'right'
 
     def __post_init__(self):
-        if not isinstance(self.data, (list, np.ndarray, pd.DataFrame)):
+        if not isinstance(self.data, (list, str, np.ndarray, pd.DataFrame)):
             raise AttributeError(
                 "data doit être un array-like (list, NumPy array ou "
                 f"DataFrame) et non de type {type(self.data)}."
@@ -101,7 +102,7 @@ class RollingWindow:
                 return self._out_of_lower_band(idx, 1)
             else:
                 return (
-                    self.data[idx-self.window+1:idx+1].copy()
+                    deepcopy(self.data[idx-self.window+1:idx+1])
                     if not self.pandas else
                     (
                         self.data
@@ -114,7 +115,7 @@ class RollingWindow:
                 return self._out_of_upper_band(idx, 0)
             else:
                 return (
-                    self.data[idx:idx+self.window].copy()
+                    deepcopy(self.data[idx:idx+self.window])
                     if not self.pandas else
                     (
                         self.data
@@ -131,7 +132,7 @@ class RollingWindow:
                 return self._out_of_upper_band(idx, (self.window-1)//2)
             else:
                 return (
-                    self.data[idx-middle_win:idx+right_res].copy()
+                    deepcopy(self.data[idx-middle_win:idx+right_res])
                     if not self.pandas else
                     (
                         self.data
@@ -155,7 +156,7 @@ class RollingWindow:
             roll = roll.astype(self.data.dtypes.to_dict())
             return roll
         elif isinstance(self.data, np.ndarray):
-            roll = np.empty((self.window, self.data.shape[1]))
+            roll = np.empty((self.window, *self.data.shape[1:]))
             roll.fill(np.nan)
             roll[self.window-idx-offset:self.window] = (
                 self.data[0:idx+offset]
@@ -179,7 +180,7 @@ class RollingWindow:
             roll = roll.astype(self.data.dtypes.to_dict())
             return roll
         elif isinstance(self.data, np.ndarray):
-            roll = np.empty((self.window, self.data.shape[1]))
+            roll = np.empty((self.window, *self.data.shape[1:]))
             roll.fill(np.nan)
             roll[0:len(self.data)-idx+offset] = self.data[idx-offset:]
             return roll
