@@ -14,7 +14,7 @@ import random
 import numpy as np
 
 
-def set_seed(seed: int):
+def set_seed(seed: int, cuda: bool=True):
     r"""
     Fonction permettant d'initialiser tous
     les générateurs pseudo-aléatoire (Python +
@@ -25,14 +25,17 @@ def set_seed(seed: int):
     ----------
     seed: int
         Racine souhaitée.
+    cuda: bool (=True)
+        Indique si l'initialisation doit être fait sur le backend cuda
+        si celui-ci est disponible ou non.
     """
     random.seed(seed)
-    np.random.seed(seed)
+    # la seed de numpy est de précision 2**32
+    np.random.seed(seed % 2**32)
     try:
         import torch
         torch.manual_seed(seed)
-        if torch.cuda.is_available() > 0:
+        if torch.cuda.is_available() > 0 and cuda:
             torch.cuda.manual_seed_all(seed)
     except ModuleNotFoundError:
         pass
-
